unit VRegistry;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Registry;

type
   TRegEntryType = (etBool, etCurrency, etDate,
                    etDateTime, etFloat, etInteger, etString,
                    etTime);

   TRegEntry = class(TCollectionItem)
   private
      FName : string;
      FSubKey: string;
      FInitValue: string;
      FValue : Variant;
      FType : TRegEntryType;
   protected
      procedure SetValue(Value : Variant);
      procedure SetName(Value : string);
      procedure SetSubKey( Value : string);
      procedure SetInitValue( Value : string);
   public
      property Value: Variant read FValue write SetValue;
      constructor Create(Collection: TCollection); override;
   published
      property Name: string read FName write SetName;
      property SubKey: string read FSubKey write SetSubKey;
      property DefaultValue: string read FInitValue write SetInitValue;
      property EntryType : TRegEntryType read FType write FType;
   end;

   TRegEntries = class(TCollection)
   private
      FOwner: TComponent;
   protected
      function GetOwner: TPersistent; override;
   public
      function Add: TRegEntry;
      function Insert( Index: Integer): TRegEntry;
      function FindItemID( ID: Integer): TRegEntry;
      function GetItem( Index: Integer): TRegEntry;
      procedure SetItem( Index: Integer; src: TRegEntry);
      constructor Create( Owner: TComponent);
      destructor Destroy; override;
      property Items[ Index: Integer]: TRegEntry read GetItem write SetItem;
   end;


  TVRegistry = class(TComponent)
  private
     Registry : TRegistry;
     FItems : TRegEntries;
     FRootKey : string;
     function ReadValue(Entry : TRegEntry):Variant;
     procedure WriteValue(Entry : TRegEntry; Value : Variant);
    { Private declarations }
  protected
    { Protected declarations }
    procedure Loaded; override;
  public
    { Public declarations }
      constructor Create( Owner: TComponent); override;
      destructor Destroy; override;
      function GetValue( EntryName : string; SubKey : string = ''):Variant;
      procedure SetValue( EntryName:string; Value : Variant; SubKey : string = '');
  published
      property Items : TRegEntries read FItems write FItems;
      property RootKey : string read FRootKey write FRootKey stored true;
    { Published declarations }
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Samples', [TVRegistry]);
end;

{ TRegEntries }

function TRegEntries.Add: TRegEntry;
begin
   Result := TRegEntry( inherited Add);
end;

constructor TRegEntries.Create(Owner: TComponent);
begin
   inherited Create(TRegEntry);
   FOwner := Owner;
end;

destructor TRegEntries.Destroy;
begin
   FOwner := nil;
   inherited Destroy;
end;

function TRegEntries.FindItemID(ID: Integer): TRegEntry;
begin
   Result := TRegEntry( inherited FindItemID( ID));
end;

function TRegEntries.GetItem(Index: Integer): TRegEntry;
begin
   Result := TRegEntry( inherited GetItem( Index));
end;

function TRegEntries.GetOwner: TPersistent;
begin
   Result := FOwner;
end;

function TRegEntries.Insert(Index: Integer): TRegEntry;
begin
   Result := TRegEntry( inherited Insert( Index));
end;

procedure TRegEntries.SetItem(Index: Integer; src: TRegEntry);
begin
   inherited SetItem( Index, src);
end;

{ TRegEntry }

constructor TRegEntry.Create(Collection: TCollection);
begin
   inherited Create(Collection);
   FType := etString;
end;

procedure TRegEntry.SetInitValue(Value: string);
begin
   FInitValue := Value;
end;

procedure TRegEntry.SetName(Value: string);
begin
   FName := Value;
end;

procedure TRegEntry.SetSubKey(Value: string);
begin
   FSubKey := Value;
end;

procedure TRegEntry.SetValue(Value: Variant);
begin
   FValue := Value;
end;

{ TVRegistry }

constructor TVRegistry.Create(Owner: TComponent);
begin
   inherited Create(Owner);
   FItems := TRegEntries.Create(Self);
   Registry := TRegistry.Create;
end;

destructor TVRegistry.Destroy;
begin
   Registry.Free;
   FItems.Free;
   inherited Destroy;
end;

function TVRegistry.GetValue(EntryName: string; SubKey : string): Variant;
var
  i : Integer;
  Entry : TRegEntry;
begin
  Result := '';
  for i:=1 to FItems.Count do
   begin
      Entry := FItems.Items[i-1];
      if(SubKey <> '') then
      begin
      if(StrIComp(PChar(Entry.SubKey), PChar(SubKey))=0)then
      if(StrIComp(PChar(Entry.Name), PChar(EntryName))=0)then
      begin
         Result := Entry.Value;
         break;
      end;
      end
      else
      if(StrIComp(PChar(Entry.Name), PChar(EntryName))=0)then
      begin
         Result := Entry.Value;
         break;         
      end;
   end;
end;

procedure TVRegistry.Loaded;
var
   i : Integer;
   Entry : TRegEntry;
begin
   inherited Loaded;
   Registry.OpenKey(RootKey, true);
   Registry.CloseKey;
   for i:=1 to FItems.Count do
   begin
      Entry := FItems.Items[i-1];
      if not Registry.KeyExists(RootKey + '\' + Entry.SubKey) then
      begin
         WriteValue(Entry,Entry.DefaultValue);
      end;
      Entry.Value := ReadValue(Entry);
   end;
end;

function TVRegistry.ReadValue(Entry: TRegEntry): Variant;
begin
   Registry.OpenKey(RootKey + '\' + Entry.SubKey,true);
   case Entry.EntryType of
      etBool:  Result := Variant(Registry.ReadBool(Entry.Name));
      etCurrency:  Result := Variant(Registry.ReadCurrency(Entry.Name));
      etDate:  Result := Variant(Registry.ReadDate(Entry.Name));
      etDateTime:  Result := Variant(Registry.ReadDateTime(Entry.Name));
      etFloat:  Result := Variant(Registry.ReadFloat(Entry.Name));
      etInteger:  Result := Variant(Registry.ReadInteger(Entry.Name));
      etString:  Result := Variant(Registry.ReadString(Entry.Name));
      etTime:  Result := Variant(Registry.ReadTime(Entry.Name));
    end;
   Registry.CloseKey;
end;

procedure TVRegistry.SetValue(EntryName : string; Value: variant; SubKey : string);
var
   i : Integer;
   Entry : TRegEntry;
begin
  for i:=1 to FItems.Count do
   begin
      Entry := FItems.Items[i-1];
      if(SubKey <> '') then
      begin
      if(StrIComp(PChar(Entry.SubKey), PChar(SubKey))=0)then
      if(StrIComp(PChar(Entry.Name), PChar(EntryName))=0)then
      begin
         Entry.Value := Value;
         WriteValue(Entry,Value);
         break;
      end;
      end
      else
      if(StrIComp(PChar(Entry.Name), PChar(EntryName))=0)then
      begin
         Entry.Value := Value;
         WriteValue(Entry,Value);
         break;
      end;
   end;
end;

procedure TVRegistry.WriteValue(Entry: TRegEntry; Value: Variant);
begin
   Registry.OpenKey(RootKey + '\' + Entry.SubKey,true);
   case Entry.EntryType of
      etBool:  Registry.WriteBool(Entry.Name,Boolean(Value));
      etCurrency:  Registry.WriteCurrency(Entry.Name,Currency(Value));
      etDate:  Registry.WriteDate(Entry.Name,TDateTime(Value));
      etDateTime:  Registry.WriteDateTime(Entry.Name,TDateTime(Value));
      etFloat:  Registry.WriteFloat(Entry.Name,Double(Value));
      etInteger:  Registry.WriteInteger(Entry.Name,Integer(Value));
      etString:  Registry.WriteString(Entry.Name,String(Value));
      etTime:  Registry.WriteTime(Entry.Name,TDateTime(Value));
    end;
   Registry.CloseKey;
end;

end.
